////////////////////////////////////////////////////////////////////////////
//
// USB LEDCg.
//
// Copyright by rubidesu 2017.
// http://www.gatelink.co.jp/hw/
//
// 2015.08.30 V1.00.
// 2015.09.01 V1.10.
// 2017.03.17 V1.20.
//
////////////////////////////////////////////////////////////////////////////

#include <12f683.h>
#include "picdef.h"

//#define SERIAL_DEBUG
#undef SERIAL_DEBUG

#rom 0x7FF = {0x3400}
#use fast_io(A)

//}CR̃q[Yݒ.
#fuses INTRC_IO, NOWDT, PUT, NOPROTECT, NOMCLR, BROWNOUT

//fBCp̃NbNݒ.
#use delay(clock=8000000)

//fobOo͂VA|[g̐ݒ.
#use RS232(BAUD=9600, XMIT=PIN_A0)

//PWM֌W̃WX^.
#byte CCPR1L = 0x13
#byte CCP1CON = 0x15

//X^oCLED̃|[g.
#define SLEEP_LED		PORTA.bit1

//[^GR[_1NbNPWMl.
#define INC_DEC_STEP	4
#define MAX_DUTY		100

////////////////////////////////////////////////////////////////////////////
void serial_out(BYTE bChar)
{
	putc(bChar);
}

////////////////////////////////////////////////////////////////////////////
void print_word10(WORD val)
{
	putc((val / 1000) + '0');
	val %= 1000;

	putc((val / 100) + '0');
	val %= 100;

	putc((val / 10) + '0');
	val %= 10;

	putc(val + '0');

	putc('\r');
	putc('\n');
}

////////////////////////////////////////////////////////////////////////////
void set_pwm_value(WORD wVal)
{
	if(wVal > MAX_DUTY){
		wVal = MAX_DUTY;
	}

	CCPR1L = (BYTE)wVal;

#ifdef SERIAL_DEBUG
	print_word10(wVal);
#endif //SERIAL_DEBUG
}

////////////////////////////////////////////////////////////////////////////
// C.
//
// IN:
//
// OUT:
//
// RETURN:
//
////////////////////////////////////////////////////////////////////////////
main()
{
	BYTE bNowEncoder;
	BYTE bLastEncoder = 0xff;
	BYTE bCalcEncoder;
	WORD wNowPwmVal = 0;

	setup_oscillator(OSC_8MHZ);
	set_tris_a(0b00111000);
	port_a_pullups(0b00110000);

	//PICW[̐ݒ.
	setup_comparator(NC_NC_NC_NC);		//Rp[^.
	setup_adc(adc_off);					//AD.
	setup_adc_ports(NO_ANALOGS);		//AD.

	//PWM̐ݒ.
	CCP1CON = CCP_PWM;
	setup_timer_2(T2_DIV_BY_1, MAX_DUTY - 1, 1);

	set_pwm_value(0x00);

	SLEEP_LED = HIGH;

	delay_ms(100);

	while(1){
		bNowEncoder = (PORTA_PORT & 0x30);
		delay_ms(1);
		if(bNowEncoder != (PORTA_PORT & 0x30)){
			continue;
		}
		bCalcEncoder = (bLastEncoder & 0xf0) | (bNowEncoder >> 4);
		bLastEncoder = bNowEncoder;

		switch(bCalcEncoder){
		//case 0x01:
		//case 0x13:
		//case 0x20:
		case 0x32:
			if(wNowPwmVal < MAX_DUTY){
				//putc('+');
				//printf(serial_out, "%02X\r\n", bCalcEncoder);

				wNowPwmVal = wNowPwmVal + INC_DEC_STEP;

				set_pwm_value(wNowPwmVal);

				if(wNowPwmVal >= MAX_DUTY){
					SLEEP_LED = HIGH;
				}
				else{
					SLEEP_LED = LOW;
				}
			}
			break;
		//case 0x02:
		//case 0x10:
		//case 0x23:
		case 0x31:
			if(wNowPwmVal >= INC_DEC_STEP){
				//putc('-');
				//printf(serial_out, "%02X\r\n", bCalcEncoder);

				wNowPwmVal = wNowPwmVal - INC_DEC_STEP;

				set_pwm_value(wNowPwmVal);

				if(wNowPwmVal == 0){
					SLEEP_LED = HIGH;
				}
				else{
					SLEEP_LED = LOW;
				}
			}
			break;
		default:
			break;
		}
	}
}

